package de.hft.stuttgart.citydoctor2.healing;

import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hft.stuttgart.citydoctor2.check.CheckId;
import de.hft.stuttgart.citydoctor2.check.CheckResult;
import de.hft.stuttgart.citydoctor2.check.ErrorId;
import de.hft.stuttgart.citydoctor2.check.HealingID;
import de.hft.stuttgart.citydoctor2.check.HealingMethod;
import de.hft.stuttgart.citydoctor2.check.ModificationListener;
import de.hft.stuttgart.citydoctor2.check.ResultStatus;
import de.hft.stuttgart.citydoctor2.check.error.NonPlanarPolygonDistancePlaneError;
import de.hft.stuttgart.citydoctor2.connect.edge.CppFeature;
import de.hft.stuttgart.citydoctor2.connect.edge.CppHealResult;
import de.hft.stuttgart.citydoctor2.connect.edge.CppPolygonHealing;
import de.hft.stuttgart.citydoctor2.datastructure.Geometry;
import de.hft.stuttgart.citydoctor2.datastructure.Polygon;

public class HealPlanarPolygonCpp implements HealingMethod {
	
	private static final Logger logger = LogManager.getLogger(HealPlanarPolygonCpp.class);

	@Override
	public HealingMethod createNew() {
		return new HealPlanarPolygonCpp();
	}

	@Override
	public boolean visit(NonPlanarPolygonDistancePlaneError err, ModificationListener l) {
		logger.debug("Executing Repair for NonPlanarPolygonDistancePlaneError");
		// ----------------------------------------------
		// get Geometry and Geometry informations
		// ----------------------------------------------
		Geometry geometry;
		if (err.getPolygon().isLinkedTo()) {
			geometry = err.getPolygon().getLinkedFromPolygon().getParent();
		} else {
			geometry = err.getPolygon().getParent();
		}

		List<Integer> polyIdsList = new ArrayList<>();
		for (int i = 0; i < geometry.getPolygons().size(); i++) {
			Polygon p = geometry.getPolygons().get(i);
			CheckResult cr = p.getCheckResult(CheckId.C_GE_P_NON_PLANAR);
			if (cr.getResultStatus() == ResultStatus.ERROR
					&& cr.getError().getErrorId() == ErrorId.GE_P_NON_PLANAR_POLYGON_DISTANCE_PLANE) {
				polyIdsList.add(i);
			}
		}
		
		int[] polyIds = new int[polyIdsList.size()];
		for (int i = 0; i < polyIdsList.size(); i++) {
			polyIds[i] = polyIdsList.get(i);
		}
		
		String gmlId_string = geometry.getParent().getGmlId().toString();

		// ----------------------------------------------
		// create feature and fill feature with geometry
		// ----------------------------------------------

		CppFeature feature = new CppFeature(gmlId_string);

		feature.setGeometry(geometry);
		feature.setLoD(geometry.getLod());

		// ----------------------------------------------
		// start healing method
		// ----------------------------------------------

		CppPolygonHealing cppPolygonHealing = new CppPolygonHealing(feature);
		cppPolygonHealing.createCppObject();

		CppHealResult healResult = cppPolygonHealing.healPlanarity(polyIds);

		if(healResult.isEmpty())
			return false;
		
		healResult.mergeIntoGeometry(geometry);

		// ----------------------------------------------
		// dispose cpp Objects
		// ----------------------------------------------

		cppPolygonHealing.disposeCppObject();
		feature.disposeCppObject();

		return true; 
	}

	@Override
	public HealingID getID() {
		return HealingID.P_NON_PLANAR_POLYGON_CPP;
	}

}
